/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Jul 22, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.coreservices.osservices.CSIdentityQuery;

//import dstep.AvailabilityMacros;
import dstep.corefoundation.CFArray;
import dstep.corefoundation.CFBase;
import dstep.corefoundation.CFData;
import dstep.corefoundation.CFError;
import dstep.corefoundation.CFRunLoop;
import dstep.corefoundation.CFUUID;
import dstep.coreservices.osservices.CSIdentity;
import dstep.coreservices.osservices.CSIdentityAuthority;

alias uint CSIdentityQueryFlags;
alias int CSIdentityQueryStringComparisonMethod;
alias int CSIdentityQueryEvent;

extern (C)
{
	alias void function (CSIdentityQueryRef, int, CFArrayRef, CFErrorRef, void*) CSIdentityQueryReceiveEventCallback;
}

enum
{
	kCSIdentityQueryGenerateUpdateEvents = 0x0001,
	kCSIdentityQueryIncludeHiddenIdentities = 0x0002
}

enum
{
	kCSIdentityQueryStringEquals = 1,
	kCSIdentityQueryStringBeginsWith = 2
}

enum
{
	kCSIdentityQueryEventSearchPhaseFinished = 1,
	kCSIdentityQueryEventResultsAdded = 2,
	kCSIdentityQueryEventResultsChanged = 3,
	kCSIdentityQueryEventResultsRemoved = 4,
	kCSIdentityQueryEventErrorOccurred = 5
}

struct CSIdentityQueryClientContext
{
	int version_;
	void* info;
	CFAllocatorRetainCallBack retainInfo;
	CFAllocatorReleaseCallBack releaseInfo;
	CFAllocatorCopyDescriptionCallBack copyInfoDescription;
	CSIdentityQueryReceiveEventCallback receiveEvent;
}

extern (C)
{
	uint CSIdentityQueryGetTypeID ();
	CSIdentityQueryRef CSIdentityQueryCreate (CFAllocatorRef allocator, int identityClass, CSIdentityAuthorityRef authority);
	CSIdentityQueryRef CSIdentityQueryCreateForName (CFAllocatorRef allocator, CFStringRef name, int comparisonMethod, int identityClass, CSIdentityAuthorityRef authority);
	CSIdentityQueryRef CSIdentityQueryCreateForUUID (CFAllocatorRef allocator, CFUUIDRef uuid, CSIdentityAuthorityRef authority);
	CSIdentityQueryRef CSIdentityQueryCreateForPosixID (CFAllocatorRef allocator, uint posixID, int identityClass, CSIdentityAuthorityRef authority);
	CSIdentityQueryRef CSIdentityQueryCreateForPersistentReference (CFAllocatorRef allocator, CFDataRef referenceData);
	CSIdentityQueryRef CSIdentityQueryCreateForCurrentUser (CFAllocatorRef allocator);
	CFArrayRef CSIdentityQueryCopyResults (CSIdentityQueryRef query);
	ubyte CSIdentityQueryExecute (CSIdentityQueryRef query, uint flags, CFErrorRef* error);
	ubyte CSIdentityQueryExecuteAsynchronously (CSIdentityQueryRef query, uint flags, CSIdentityQueryClientContext* clientContext, CFRunLoopRef runLoop, CFStringRef runLoopMode);
	void CSIdentityQueryStop (CSIdentityQueryRef query);
}